from anthropic import Anthropic, AsyncAnthropic

from .base import AsyncTokenizer, Tokenizer
from ..providers.anthropic_ai import AnthropicProvider, AsyncAnthropicProvider
from ..model import ProviderName, TokenizerResponse


class AnthropicTokenizer(Tokenizer):
    def __init__(self, model: str):
        super().__init__(model)
        self.client = Anthropic()
        self.provider = AnthropicProvider()

    def count_tokens(self, text: str) -> TokenizerResponse:
        if self.model not in self.provider.models():
            raise ValueError(f'Unsupported model: {self.model}')
        count = self.client.messages.count_tokens(
            model=self.model,
            messages=[
                {'role': 'user', 'content': text},
            ],
        )
        return TokenizerResponse(
            input_tokens=count.input_tokens,
            model=self.model,
            provider=ProviderName.ANTHROPIC.value,
        )


class AsyncAnthropicTokenizer(AsyncTokenizer):
    def __init__(self, model: str):
        super().__init__(model)
        self.client = AsyncAnthropic()
        self.provider = AsyncAnthropicProvider()

    async def count_tokens(self, text: str) -> TokenizerResponse:
        if self.model not in await self.provider.models():
            raise ValueError(f'Unsupported model: {self.model}')
        count = await self.client.messages.count_tokens(
            model=self.model,
            messages=[
                {'role': 'user', 'content': text},
            ],
        )
        return TokenizerResponse(
            input_tokens=count.input_tokens,
            model=self.model,
            provider=ProviderName.ANTHROPIC.value,
        )



import pytest
from unittest.mock import MagicMock, AsyncMock

from tokemon.tokenizers.xai import (
    XaiTokenizer,
    AsyncXaiTokenizer,
)
from tokemon.model import ProviderName, TokenizerResponse


FAKE_MODELS = ["grok-3", "grok-3-mini", "grok-2-vision-1212"]


@pytest.fixture
def valid_model():
    return FAKE_MODELS[0]


@pytest.fixture
def mock_sync_provider(monkeypatch):
    mock_prov = MagicMock()
    mock_prov.models.return_value = FAKE_MODELS

    monkeypatch.setattr(
        "tokemon.tokenizers.xai.XaiProvider",
        lambda: mock_prov,
    )

    return mock_prov


@pytest.fixture
def mock_async_provider(monkeypatch):
    mock_prov = MagicMock()
    mock_prov.models = AsyncMock(return_value=FAKE_MODELS)

    monkeypatch.setattr(
        "tokemon.tokenizers.xai.AsyncXaiProvider",
        lambda: mock_prov,
    )

    return mock_prov


@pytest.fixture
def mock_sync_xai_client(monkeypatch):
    mock_client = MagicMock()

    mock_client.tokenize.tokenize_text.return_value = ["t1", "t2", "t3"]

    monkeypatch.setattr(
        "tokemon.tokenizers.xai.Client",
        lambda: mock_client,
    )

    return mock_client


@pytest.fixture
def mock_async_xai_client(monkeypatch):
    mock_client = MagicMock()

    mock_client.tokenize.tokenize_text = AsyncMock(
        return_value=["t1", "t2", "t3", "t4"]
    )

    monkeypatch.setattr(
        "tokemon.tokenizers.xai.AsyncClient",
        lambda: mock_client,
    )

    return mock_client


def test_sync_init_with_valid_model(
    valid_model, mock_sync_provider, mock_sync_xai_client
):
    tokenizer = XaiTokenizer(valid_model)

    assert tokenizer.model == valid_model
    assert tokenizer.client is mock_sync_xai_client


def test_async_init_with_valid_model(
    valid_model, mock_async_provider, mock_async_xai_client
):
    tokenizer = AsyncXaiTokenizer(valid_model)

    assert tokenizer.model == valid_model
    assert tokenizer.client is mock_async_xai_client


def test_sync_count_tokens_with_invalid_model(
    mock_sync_provider, mock_sync_xai_client
):
    tokenizer = XaiTokenizer("invalid-model")

    with pytest.raises(ValueError, match="Unsupported model"):
        tokenizer.count_tokens("hello")


@pytest.mark.asyncio
async def test_async_count_tokens_with_invalid_model(
    mock_async_provider, mock_async_xai_client
):
    tokenizer = AsyncXaiTokenizer("invalid-model")

    with pytest.raises(ValueError, match="Unsupported model"):
        await tokenizer.count_tokens("hello")


def test_sync_count_tokens_normal_input(
    valid_model, mock_sync_provider, mock_sync_xai_client
):
    tokenizer = XaiTokenizer(valid_model)

    response = tokenizer.count_tokens("hello")

    assert isinstance(response, TokenizerResponse)
    assert response.input_tokens == 3
    assert response.model == valid_model
    assert response.provider == ProviderName.XAI.value

    mock_sync_xai_client.tokenize.tokenize_text.assert_called_once_with(
        model=valid_model,
        text="hello",
    )


def test_sync_count_tokens_empty_string(
    valid_model, mock_sync_provider, mock_sync_xai_client
):
    mock_sync_xai_client.tokenize.tokenize_text.return_value = []

    tokenizer = XaiTokenizer(valid_model)
    response = tokenizer.count_tokens("")

    assert response.input_tokens == 0


def test_sync_count_tokens_large_input(
    valid_model, mock_sync_provider, mock_sync_xai_client
):
    mock_sync_xai_client.tokenize.tokenize_text.return_value = ["t"] * 10_000

    tokenizer = XaiTokenizer(valid_model)
    response = tokenizer.count_tokens("large input")

    assert response.input_tokens == 10_000


@pytest.mark.asyncio
async def test_async_count_tokens_normal_input(
    valid_model, mock_async_provider, mock_async_xai_client
):
    tokenizer = AsyncXaiTokenizer(valid_model)

    response = await tokenizer.count_tokens("hello async")

    assert isinstance(response, TokenizerResponse)
    assert response.input_tokens == 4
    assert response.model == valid_model
    assert response.provider == ProviderName.XAI.value

    mock_async_xai_client.tokenize.tokenize_text.assert_awaited_once_with(
        model=valid_model,
        text="hello async",
    )


@pytest.mark.asyncio
async def test_async_count_tokens_empty_string(
    valid_model, mock_async_provider, mock_async_xai_client
):
    mock_async_xai_client.tokenize.tokenize_text.return_value = []

    tokenizer = AsyncXaiTokenizer(valid_model)
    response = await tokenizer.count_tokens("")

    assert response.input_tokens == 0



from google import genai

from .base import AsyncTokenizer, Tokenizer
from ..providers.google_ai import GoogleProvider, AsyncGoogleProvider
from ..model import ProviderName, TokenizerResponse


class GoogleAITokenizer(Tokenizer):
    def __init__(self, model: str):
        super().__init__(model)
        self.client = genai.Client()
        self.provider = GoogleProvider()

    def count_tokens(self, text: str) -> TokenizerResponse:
        if self.model not in self.provider.models():
            raise ValueError(f'Unsupported model: {self.model}')
        response = self.client.models.count_tokens(
            model=self.model,
            contents=text,
        )
        return TokenizerResponse(
            input_tokens=response.total_tokens,
            model=self.model,
            provider=ProviderName.GOOGLE.value,
        )


class AsyncGoogleAITokenizer(AsyncTokenizer):
    def __init__(self, model: str):
        super().__init__(model)
        self.client = genai.Client()
        self.provider = AsyncGoogleProvider()

    async def count_tokens(self, text: str) -> TokenizerResponse:
        if self.model not in await self.provider.models():
            raise ValueError(f'Unsupported model: {self.model}')
        response = await self.client.aio.models.count_tokens(
            model=self.model,
            contents=text,
        )
        return TokenizerResponse(
            input_tokens=response.total_tokens,
            model=self.model,
            provider=ProviderName.GOOGLE.value,
        )



from functools import lru_cache

from async_lru import alru_cache
from google import genai

from .base import Provider, AsyncProvider


def _strip_models_prefix(name: str) -> str:
    if name.startswith('models/'):
        return name[len('models/'):]
    return name


class GoogleProvider(Provider):
    def __init__(self):
        self.client = genai.Client()

    @lru_cache(maxsize=1)
    def models(self) -> list[str]:
        client = genai.Client()
        return [_strip_models_prefix(m.name) for m in client.models.list()]


class AsyncGoogleProvider(AsyncProvider):
    def __init__(self):
        self.client = genai.Client()

    @alru_cache(maxsize=1, ttl=300)
    async def models(self) -> list[str]:
        response = await self.client.aio.models.list()
        return [
            _strip_models_prefix(m.name) for m in response
        ]



from functools import lru_cache

import tiktoken

from .base import Provider


class OpenAIProvider(Provider):
    @lru_cache(maxsize=1)
    def models(self) -> list[str]:
        return list(tiktoken.model.MODEL_TO_ENCODING.keys())



from dataclasses import dataclass
from enum import Enum


class Mode:
    SYNC = 'sync'
    ASYNC = 'async'


class ProviderName(Enum):
    OPENAI = 'openai'
    ANTHROPIC = 'anthropic'
    XAI = 'xai'
    GOOGLE = 'google'


@dataclass
class TokenizerResponse:
    input_tokens: int | None
    model: str
    provider: str



import pytest
from unittest.mock import MagicMock, AsyncMock

from tokemon.tokenizers.google_ai import (
    GoogleAITokenizer,
    AsyncGoogleAITokenizer,
)
from tokemon.model import ProviderName, TokenizerResponse


FAKE_MODELS = ["gemini-2.5-pro", "gemini-2.5-flash", "gemini-2.0-flash"]


@pytest.fixture
def valid_model():
    return FAKE_MODELS[0]


@pytest.fixture
def mock_sync_provider(monkeypatch):
    mock_prov = MagicMock()
    mock_prov.models.return_value = FAKE_MODELS

    monkeypatch.setattr(
        "tokemon.tokenizers.google_ai.GoogleProvider",
        lambda: mock_prov,
    )

    return mock_prov


@pytest.fixture
def mock_async_provider(monkeypatch):
    mock_prov = MagicMock()
    mock_prov.models = AsyncMock(return_value=FAKE_MODELS)

    monkeypatch.setattr(
        "tokemon.tokenizers.google_ai.AsyncGoogleProvider",
        lambda: mock_prov,
    )

    return mock_prov


@pytest.fixture
def mock_google_client(monkeypatch):
    mock_client = MagicMock()

    # Sync path
    mock_client.models.count_tokens.return_value = MagicMock(
        total_tokens=5
    )

    # Async path
    mock_client.aio = MagicMock()
    mock_client.aio.models.count_tokens = AsyncMock(
        return_value=MagicMock(total_tokens=7)
    )

    monkeypatch.setattr(
        "tokemon.tokenizers.google_ai.genai.Client",
        lambda: mock_client,
    )

    return mock_client


def test_sync_init_with_valid_model(
    valid_model, mock_sync_provider, mock_google_client
):
    tokenizer = GoogleAITokenizer(valid_model)

    assert tokenizer.model == valid_model
    assert tokenizer.client is mock_google_client


def test_async_init_with_valid_model(
    valid_model, mock_async_provider, mock_google_client
):
    tokenizer = AsyncGoogleAITokenizer(valid_model)

    assert tokenizer.model == valid_model
    assert tokenizer.client is mock_google_client


def test_sync_count_tokens_with_invalid_model(
    mock_sync_provider, mock_google_client
):
    tokenizer = GoogleAITokenizer("invalid-model")

    with pytest.raises(ValueError, match="Unsupported model"):
        tokenizer.count_tokens("hello")


@pytest.mark.asyncio
async def test_async_count_tokens_with_invalid_model(
    mock_async_provider, mock_google_client
):
    tokenizer = AsyncGoogleAITokenizer("invalid-model")

    with pytest.raises(ValueError, match="Unsupported model"):
        await tokenizer.count_tokens("hello")


def test_sync_count_tokens_normal_input(
    valid_model, mock_sync_provider, mock_google_client
):
    tokenizer = GoogleAITokenizer(valid_model)

    response = tokenizer.count_tokens("hello")

    assert isinstance(response, TokenizerResponse)
    assert response.input_tokens == 5
    assert response.model == valid_model
    assert response.provider == ProviderName.GOOGLE.value

    mock_google_client.models.count_tokens.assert_called_once_with(
        model=valid_model,
        contents="hello",
    )


def test_sync_count_tokens_empty_string(
    valid_model, mock_sync_provider, mock_google_client
):
    mock_google_client.models.count_tokens.return_value.total_tokens = 0

    tokenizer = GoogleAITokenizer(valid_model)
    response = tokenizer.count_tokens("")

    assert response.input_tokens == 0


def test_sync_count_tokens_large_input(
    valid_model, mock_sync_provider, mock_google_client
):
    mock_google_client.models.count_tokens.return_value.total_tokens = 10_000

    tokenizer = GoogleAITokenizer(valid_model)
    response = tokenizer.count_tokens("large input")

    assert response.input_tokens == 10_000


@pytest.mark.asyncio
async def test_async_count_tokens_normal_input(
    valid_model, mock_async_provider, mock_google_client
):
    tokenizer = AsyncGoogleAITokenizer(valid_model)

    response = await tokenizer.count_tokens("hello async")

    assert isinstance(response, TokenizerResponse)
    assert response.input_tokens == 7
    assert response.model == valid_model
    assert response.provider == ProviderName.GOOGLE.value

    mock_google_client.aio.models.count_tokens.assert_awaited_once_with(
        model=valid_model,
        contents="hello async",
    )


@pytest.mark.asyncio
async def test_async_count_tokens_empty_string(
    valid_model, mock_async_provider, mock_google_client
):
    mock_google_client.aio.models.count_tokens.return_value.total_tokens = 0

    tokenizer = AsyncGoogleAITokenizer(valid_model)
    response = await tokenizer.count_tokens("")

    assert response.input_tokens == 0



import pytest
from unittest.mock import MagicMock

from tokemon import tokemon, tokemon_models
from tokemon.model import ProviderName, Mode


@pytest.fixture
def mock_tokenizers(monkeypatch):
    mocks = {}

    def make_mock(name):
        mock_cls = MagicMock(name=name)
        mock_instance = MagicMock(name=f"{name}Instance")
        mock_cls.return_value = mock_instance
        mocks[name] = mock_cls
        return mock_cls

    monkeypatch.setattr(
        "tokemon.scaffold.OpenAITokenizer",
        make_mock("OpenAITokenizer"),
    )
    monkeypatch.setattr(
        "tokemon.scaffold.AnthropicTokenizer",
        make_mock("AnthropicTokenizer"),
    )
    monkeypatch.setattr(
        "tokemon.scaffold.AsyncAnthropicTokenizer",
        make_mock("AsyncAnthropicTokenizer"),
    )
    monkeypatch.setattr(
        "tokemon.scaffold.GoogleAITokenizer",
        make_mock("GoogleAITokenizer"),
    )
    monkeypatch.setattr(
        "tokemon.scaffold.AsyncGoogleAITokenizer",
        make_mock("AsyncGoogleAITokenizer"),
    )
    monkeypatch.setattr(
        "tokemon.scaffold.XaiTokenizer",
        make_mock("XaiTokenizer"),
    )
    monkeypatch.setattr(
        "tokemon.scaffold.AsyncXaiTokenizer",
        make_mock("AsyncXaiTokenizer"),
    )

    return mocks


@pytest.fixture
def mock_providers(monkeypatch):
    mocks = {}

    def make_mock(name):
        mock_cls = MagicMock(name=name)
        mock_instance = MagicMock(name=f"{name}Instance")
        mock_cls.return_value = mock_instance
        mocks[name] = mock_cls
        return mock_cls

    monkeypatch.setattr(
        "tokemon.scaffold.OpenAIProvider",
        make_mock("OpenAIProvider"),
    )
    monkeypatch.setattr(
        "tokemon.scaffold.AnthropicProvider",
        make_mock("AnthropicProvider"),
    )
    monkeypatch.setattr(
        "tokemon.scaffold.AsyncAnthropicProvider",
        make_mock("AsyncAnthropicProvider"),
    )
    monkeypatch.setattr(
        "tokemon.scaffold.GoogleProvider",
        make_mock("GoogleProvider"),
    )
    monkeypatch.setattr(
        "tokemon.scaffold.AsyncGoogleProvider",
        make_mock("AsyncGoogleProvider"),
    )
    monkeypatch.setattr(
        "tokemon.scaffold.XaiProvider",
        make_mock("XaiProvider"),
    )
    monkeypatch.setattr(
        "tokemon.scaffold.AsyncXaiProvider",
        make_mock("AsyncXaiProvider"),
    )

    return mocks


def test_openai_sync_factory(mock_tokenizers):
    result = tokemon(
        model="gpt-4",
        provider=ProviderName.OPENAI.value,
        mode=Mode.SYNC,
    )

    mock_tokenizers["OpenAITokenizer"].assert_called_once_with(model="gpt-4")
    assert result is mock_tokenizers["OpenAITokenizer"].return_value


def test_anthropic_sync_factory(mock_tokenizers):
    result = tokemon(
        model="claude-3",
        provider=ProviderName.ANTHROPIC.value,
        mode=Mode.SYNC,
    )

    mock_tokenizers["AnthropicTokenizer"].assert_called_once_with(model="claude-3")
    assert result is mock_tokenizers["AnthropicTokenizer"].return_value


def test_google_sync_factory(mock_tokenizers):
    result = tokemon(
        model="gemini-pro",
        provider=ProviderName.GOOGLE.value,
        mode=Mode.SYNC,
    )

    mock_tokenizers["GoogleAITokenizer"].assert_called_once_with(model="gemini-pro")
    assert result is mock_tokenizers["GoogleAITokenizer"].return_value


def test_xai_sync_factory(mock_tokenizers):
    result = tokemon(
        model="grok-2",
        provider=ProviderName.XAI.value,
        mode=Mode.SYNC,
    )

    mock_tokenizers["XaiTokenizer"].assert_called_once_with(model="grok-2")
    assert result is mock_tokenizers["XaiTokenizer"].return_value


def test_anthropic_async_factory(mock_tokenizers):
    result = tokemon(
        model="claude-3",
        provider=ProviderName.ANTHROPIC.value,
        mode=Mode.ASYNC,
    )

    mock_tokenizers["AsyncAnthropicTokenizer"].assert_called_once_with(model="claude-3")
    assert result is mock_tokenizers["AsyncAnthropicTokenizer"].return_value


def test_google_async_factory(mock_tokenizers):
    result = tokemon(
        model="gemini-pro",
        provider=ProviderName.GOOGLE.value,
        mode=Mode.ASYNC,
    )

    mock_tokenizers["AsyncGoogleAITokenizer"].assert_called_once_with(model="gemini-pro")
    assert result is mock_tokenizers["AsyncGoogleAITokenizer"].return_value


def test_xai_async_factory(mock_tokenizers):
    result = tokemon(
        model="grok-2",
        provider=ProviderName.XAI.value,
        mode=Mode.ASYNC,
    )

    mock_tokenizers["AsyncXaiTokenizer"].assert_called_once_with(model="grok-2")
    assert result is mock_tokenizers["AsyncXaiTokenizer"].return_value


def test_unsupported_provider_raises():
    with pytest.raises(ValueError, match="Unsupported provider"):
        tokemon(
            model="some-model",
            provider="not-a-provider",
            mode=Mode.SYNC,
        )


def test_unsupported_async_openai_raises(mock_tokenizers):
    with pytest.raises(ValueError, match="Unsupported provider"):
        tokemon(
            model="gpt-4",
            provider=ProviderName.OPENAI.value,
            mode=Mode.ASYNC,
        )


def test_invalid_mode_treated_as_sync(mock_tokenizers):
    result = tokemon(
        model="gpt-4",
        provider=ProviderName.OPENAI.value,
        mode="not-a-real-mode",
    )

    mock_tokenizers["OpenAITokenizer"].assert_called_once_with(model="gpt-4")
    assert result is mock_tokenizers["OpenAITokenizer"].return_value


def test_tokemon_models_openai_sync(mock_providers):
    result = tokemon_models(
        provider=ProviderName.OPENAI.value,
        mode=Mode.SYNC,
    )

    mock_providers["OpenAIProvider"].assert_called_once_with()
    assert result is mock_providers["OpenAIProvider"].return_value


def test_tokemon_models_anthropic_sync(mock_providers):
    result = tokemon_models(
        provider=ProviderName.ANTHROPIC.value,
        mode=Mode.SYNC,
    )

    mock_providers["AnthropicProvider"].assert_called_once_with()
    assert result is mock_providers["AnthropicProvider"].return_value


def test_tokemon_models_anthropic_async(mock_providers):
    result = tokemon_models(
        provider=ProviderName.ANTHROPIC.value,
        mode=Mode.ASYNC,
    )

    mock_providers["AsyncAnthropicProvider"].assert_called_once_with()
    assert result is mock_providers["AsyncAnthropicProvider"].return_value


def test_tokemon_models_google_sync(mock_providers):
    result = tokemon_models(
        provider=ProviderName.GOOGLE.value,
        mode=Mode.SYNC,
    )

    mock_providers["GoogleProvider"].assert_called_once_with()
    assert result is mock_providers["GoogleProvider"].return_value


def test_tokemon_models_google_async(mock_providers):
    result = tokemon_models(
        provider=ProviderName.GOOGLE.value,
        mode=Mode.ASYNC,
    )

    mock_providers["AsyncGoogleProvider"].assert_called_once_with()
    assert result is mock_providers["AsyncGoogleProvider"].return_value


def test_tokemon_models_xai_sync(mock_providers):
    result = tokemon_models(
        provider=ProviderName.XAI.value,
        mode=Mode.SYNC,
    )

    mock_providers["XaiProvider"].assert_called_once_with()
    assert result is mock_providers["XaiProvider"].return_value


def test_tokemon_models_xai_async(mock_providers):
    result = tokemon_models(
        provider=ProviderName.XAI.value,
        mode=Mode.ASYNC,
    )

    mock_providers["AsyncXaiProvider"].assert_called_once_with()
    assert result is mock_providers["AsyncXaiProvider"].return_value


def test_tokemon_models_unsupported_provider_raises():
    with pytest.raises(ValueError, match="Unsupported provider"):
        tokemon_models(
            provider="not-a-provider",
            mode=Mode.SYNC,
        )


def test_tokemon_models_unsupported_async_openai_raises(mock_providers):
    with pytest.raises(ValueError, match="Unsupported provider"):
        tokemon_models(
            provider=ProviderName.OPENAI.value,
            mode=Mode.ASYNC,
        )



from functools import lru_cache

from async_lru import alru_cache
from xai_sdk import AsyncClient, Client

from .base import Provider, AsyncProvider


class XaiProvider(Provider):
    def __init__(self):
        self.client = Client()

    @lru_cache(maxsize=1)
    def models(self) -> list[str]:
        return [m.name for m in self.client.models.list_language_models()]


class AsyncXaiProvider(AsyncProvider):
    def __init__(self):
        self.client = AsyncClient()

    @alru_cache(maxsize=1, ttl=300)
    async def models(self) -> list[str]:
        response = await self.client.models.list_language_models()
        return [m.name for m in response]



import pytest
from unittest.mock import MagicMock, AsyncMock

from tokemon.providers.openai import OpenAIProvider
from tokemon.providers.anthropic_ai import AnthropicProvider, AsyncAnthropicProvider
from tokemon.providers.google_ai import (
    GoogleProvider,
    AsyncGoogleProvider,
    _strip_models_prefix,
)
from tokemon.providers.xai import XaiProvider, AsyncXaiProvider


def test_strip_models_prefix_with_prefix():
    assert _strip_models_prefix("models/gemini-2.5-pro") == "gemini-2.5-pro"


def test_strip_models_prefix_without_prefix():
    assert _strip_models_prefix("gemini-2.5-pro") == "gemini-2.5-pro"


def test_strip_models_prefix_empty_string():
    assert _strip_models_prefix("") == ""


def test_openai_provider_models(monkeypatch):
    fake_model_map = {"gpt-4": "cl100k_base", "gpt-3.5-turbo": "cl100k_base"}
    monkeypatch.setattr(
        "tiktoken.model.MODEL_TO_ENCODING",
        fake_model_map,
    )

    provider = OpenAIProvider()
    result = provider.models()

    assert set(result) == {"gpt-4", "gpt-3.5-turbo"}


def test_anthropic_sync_provider_models(monkeypatch):
    mock_client = MagicMock()
    mock_model_1 = MagicMock()
    mock_model_1.id = "claude-sonnet-4-5"
    mock_model_2 = MagicMock()
    mock_model_2.id = "claude-haiku-4-5"
    mock_client.models.list.return_value = MagicMock(
        data=[mock_model_1, mock_model_2]
    )

    monkeypatch.setattr(
        "tokemon.providers.anthropic_ai.Anthropic",
        lambda: mock_client,
    )

    provider = AnthropicProvider()
    result = provider.models()

    assert result == ["claude-sonnet-4-5", "claude-haiku-4-5"]


@pytest.mark.asyncio
async def test_anthropic_async_provider_models(monkeypatch):
    mock_client = MagicMock()
    mock_model_1 = MagicMock()
    mock_model_1.id = "claude-sonnet-4-5"
    mock_model_2 = MagicMock()
    mock_model_2.id = "claude-opus-4-5"
    mock_client.models.list = AsyncMock(
        return_value=MagicMock(data=[mock_model_1, mock_model_2])
    )

    monkeypatch.setattr(
        "tokemon.providers.anthropic_ai.AsyncAnthropic",
        lambda: mock_client,
    )

    provider = AsyncAnthropicProvider()
    result = await provider.models()

    assert result == ["claude-sonnet-4-5", "claude-opus-4-5"]


def test_google_sync_provider_models(monkeypatch):
    mock_client = MagicMock()
    mock_model_1 = MagicMock()
    mock_model_1.name = "models/gemini-2.5-pro"
    mock_model_2 = MagicMock()
    mock_model_2.name = "models/gemini-2.0-flash"
    mock_client.models.list.return_value = [mock_model_1, mock_model_2]

    monkeypatch.setattr(
        "tokemon.providers.google_ai.genai.Client",
        lambda: mock_client,
    )

    provider = GoogleProvider()
    result = provider.models()

    assert result == ["gemini-2.5-pro", "gemini-2.0-flash"]


@pytest.mark.asyncio
async def test_google_async_provider_models(monkeypatch):
    mock_client = MagicMock()
    mock_model_1 = MagicMock()
    mock_model_1.name = "models/gemini-2.5-flash"
    mock_model_2 = MagicMock()
    mock_model_2.name = "gemini-2.0-flash-lite"

    mock_client.aio = MagicMock()
    mock_client.aio.models.list = AsyncMock(
        return_value=[mock_model_1, mock_model_2]
    )

    monkeypatch.setattr(
        "tokemon.providers.google_ai.genai.Client",
        lambda: mock_client,
    )

    provider = AsyncGoogleProvider()
    result = await provider.models()

    assert result == ["gemini-2.5-flash", "gemini-2.0-flash-lite"]


def test_xai_sync_provider_models(monkeypatch):
    mock_client = MagicMock()
    mock_model_1 = MagicMock()
    mock_model_1.name = "grok-3"
    mock_model_2 = MagicMock()
    mock_model_2.name = "grok-3-mini"
    mock_client.models.list_language_models.return_value = [
        mock_model_1, mock_model_2
    ]

    monkeypatch.setattr(
        "tokemon.providers.xai.Client",
        lambda: mock_client,
    )

    provider = XaiProvider()
    result = provider.models()

    assert result == ["grok-3", "grok-3-mini"]


@pytest.mark.asyncio
async def test_xai_async_provider_models(monkeypatch):
    mock_client = MagicMock()
    mock_model_1 = MagicMock()
    mock_model_1.name = "grok-3"
    mock_model_2 = MagicMock()
    mock_model_2.name = "grok-2-vision-1212"

    mock_client.models.list_language_models = AsyncMock(
        return_value=[mock_model_1, mock_model_2]
    )

    monkeypatch.setattr(
        "tokemon.providers.xai.AsyncClient",
        lambda: mock_client,
    )

    provider = AsyncXaiProvider()
    result = await provider.models()

    assert result == ["grok-3", "grok-2-vision-1212"]



import tiktoken

from .base import Tokenizer
from ..providers.openai import OpenAIProvider
from ..model import ProviderName, TokenizerResponse


class OpenAITokenizer(Tokenizer):
    def __init__(self, model: str):
        super().__init__(model)
        self.provider = OpenAIProvider()

    def count_tokens(self, text: str) -> TokenizerResponse:
        if self.model not in self.provider.models():
            raise ValueError(f'Unsupported model: {self.model}')
        encoding = tiktoken.encoding_for_model(self.model)
        response = encoding.encode(text)
        return TokenizerResponse(
            input_tokens=len(response),
            model=self.model,
            provider=ProviderName.OPENAI.value,
        )



import abc


class Provider(abc.ABC):
    @abc.abstractmethod
    def models(self) -> list[str]:
        pass  # pragma: no cover


class AsyncProvider(abc.ABC):
    @abc.abstractmethod
    async def models(self) -> list[str]:
        pass  # pragma: no cover



import abc

from ..model import TokenizerResponse


class Tokenizer(abc.ABC):
    def __init__(self, model: str):
        self.model = model

    @abc.abstractmethod
    def count_tokens(self, text: str) -> TokenizerResponse:
        pass


class AsyncTokenizer(abc.ABC):
    def __init__(self, model: str):
        self.model = model

    @abc.abstractmethod
    async def count_tokens(self, text: str) -> TokenizerResponse:
        pass



from xai_sdk import AsyncClient, Client

from .base import AsyncTokenizer, Tokenizer
from ..providers.xai import XaiProvider, AsyncXaiProvider
from ..model import ProviderName, TokenizerResponse


class XaiTokenizer(Tokenizer):
    def __init__(self, model: str):
        super().__init__(model)
        self.client = Client()
        self.provider = XaiProvider()

    def count_tokens(self, text: str) -> TokenizerResponse:
        if self.model not in self.provider.models():
            raise ValueError(f'Unsupported model: {self.model}')
        response = self.client.tokenize.tokenize_text(
            model=self.model,
            text=text,
        )
        return TokenizerResponse(
            input_tokens=len(response),
            model=self.model,
            provider=ProviderName.XAI.value,
        )


class AsyncXaiTokenizer(AsyncTokenizer):
    def __init__(self, model: str):
        super().__init__(model)
        self.client = AsyncClient()
        self.provider = AsyncXaiProvider()

    async def count_tokens(self, text: str) -> TokenizerResponse:
        if self.model not in await self.provider.models():
            raise ValueError(f'Unsupported model: {self.model}')
        response = await self.client.tokenize.tokenize_text(
            model=self.model,
            text=text,
        )
        return TokenizerResponse(
            input_tokens=len(response),
            model=self.model,
            provider=ProviderName.XAI.value,
        )



from .providers.base import AsyncProvider, Provider
from .providers.anthropic_ai import AsyncAnthropicProvider, AnthropicProvider
from .providers.google_ai import AsyncGoogleProvider, GoogleProvider
from .providers.openai import OpenAIProvider
from .providers.xai import AsyncXaiProvider, XaiProvider
from .tokenizers.base import AsyncTokenizer, Tokenizer
from .tokenizers.anthropic_ai import AsyncAnthropicTokenizer, AnthropicTokenizer
from .tokenizers.google_ai import AsyncGoogleAITokenizer, GoogleAITokenizer
from .tokenizers.openai import OpenAITokenizer
from .tokenizers.xai import AsyncXaiTokenizer, XaiTokenizer
from .model import Mode, ProviderName


def tokemon(
    model: str,
    provider: str,
    mode: str = Mode.SYNC,
) -> AsyncTokenizer | Tokenizer:
    if mode == Mode.ASYNC:
        provider = f"async-{provider}"

    tokenizers: dict[str, type[AsyncTokenizer | Tokenizer]] = {
        ProviderName.OPENAI.value: OpenAITokenizer,
        ProviderName.ANTHROPIC.value: AnthropicTokenizer,
        f"async-{ProviderName.ANTHROPIC.value}": AsyncAnthropicTokenizer,
        ProviderName.XAI.value: XaiTokenizer,
        f"async-{ProviderName.XAI.value}": AsyncXaiTokenizer,
        ProviderName.GOOGLE.value: GoogleAITokenizer,
        f"async-{ProviderName.GOOGLE.value}": AsyncGoogleAITokenizer,
    }

    if provider not in tokenizers:
        raise ValueError(f"Unsupported provider: {provider}")

    return tokenizers[provider](model=model)


def tokemon_models(
    provider: str,
    mode: str = Mode.SYNC,
) -> AsyncProvider | Provider:
    if mode == Mode.ASYNC:
        provider = f'async-{provider}'

    models: dict[str, type] = {
        ProviderName.OPENAI.value: OpenAIProvider,
        ProviderName.ANTHROPIC.value: AnthropicProvider,
        f'async-{ProviderName.ANTHROPIC.value}': AsyncAnthropicProvider,
        ProviderName.XAI.value: XaiProvider,
        f'async-{ProviderName.XAI.value}': AsyncXaiProvider,
        ProviderName.GOOGLE.value: GoogleProvider,
        f'async-{ProviderName.GOOGLE.value}': AsyncGoogleProvider,
    }
    if provider not in models:
        raise ValueError(f'Unsupported provider: {provider}')

    return models[provider]()



import pytest
from unittest.mock import MagicMock, patch

from tokemon.tokenizers.openai import OpenAITokenizer
from tokemon.model import ProviderName, TokenizerResponse


FAKE_MODELS = ["gpt-4", "gpt-3.5-turbo", "gpt-4o"]


@pytest.fixture
def mock_provider(monkeypatch):
    mock_prov = MagicMock()
    mock_prov.models.return_value = FAKE_MODELS

    monkeypatch.setattr(
        "tokemon.tokenizers.openai.OpenAIProvider",
        lambda: mock_prov,
    )

    return mock_prov


@pytest.fixture
def mock_encoding(monkeypatch):
    fake_encoding = MagicMock()
    monkeypatch.setattr(
        "tiktoken.encoding_for_model",
        lambda model: fake_encoding,
    )
    return fake_encoding


@pytest.fixture
def valid_model():
    return FAKE_MODELS[0]


def test_init_with_valid_model(valid_model, mock_provider):
    tokenizer = OpenAITokenizer(valid_model)

    assert tokenizer.model == valid_model


def test_count_tokens_with_invalid_model(mock_provider, mock_encoding):
    tokenizer = OpenAITokenizer("not-a-real-model")

    with pytest.raises(ValueError, match="Unsupported model"):
        tokenizer.count_tokens("hello")


def test_count_tokens_normal_text(valid_model, mock_provider, mock_encoding):
    mock_encoding.encode.return_value = [1, 2, 3, 4]

    tokenizer = OpenAITokenizer(valid_model)
    response = tokenizer.count_tokens("hello world")

    assert isinstance(response, TokenizerResponse)
    assert response.input_tokens == 4
    assert response.model == valid_model
    assert response.provider == ProviderName.OPENAI.value

    mock_encoding.encode.assert_called_once_with("hello world")


def test_count_tokens_empty_string(valid_model, mock_provider, mock_encoding):
    mock_encoding.encode.return_value = []

    tokenizer = OpenAITokenizer(valid_model)
    response = tokenizer.count_tokens("")

    assert response.input_tokens == 0


def test_count_tokens_whitespace(valid_model, mock_provider, mock_encoding):
    mock_encoding.encode.return_value = [42]

    tokenizer = OpenAITokenizer(valid_model)
    response = tokenizer.count_tokens("   ")

    assert response.input_tokens == 1


def test_count_tokens_large_input(valid_model, mock_provider, mock_encoding):
    mock_encoding.encode.return_value = list(range(10_000))

    tokenizer = OpenAITokenizer(valid_model)
    response = tokenizer.count_tokens("large input")

    assert response.input_tokens == 10_000


def test_encode_called_exactly_once(valid_model, mock_provider, mock_encoding):
    mock_encoding.encode.return_value = [1, 2]

    tokenizer = OpenAITokenizer(valid_model)
    tokenizer.count_tokens("test")

    mock_encoding.encode.assert_called_once()



from functools import lru_cache

from anthropic import Anthropic, AsyncAnthropic
from async_lru import alru_cache

from .base import Provider, AsyncProvider


class AnthropicProvider(Provider):
    def __init__(self):
        self.client = Anthropic()

    @lru_cache(maxsize=1)
    def models(self) -> list[str]:
        client = Anthropic()
        return [m.id for m in client.models.list().data]


class AsyncAnthropicProvider(AsyncProvider):
    def __init__(self):
        self.client = AsyncAnthropic()

    @alru_cache(maxsize=1, ttl=300)
    async def models(self) -> list[str]:
        response = await self.client.models.list()
        return [m.id for m in response.data]
